(function ($, Drupal, Sortable) {
  Drupal.behaviors.media_folder_management_widget = {
    attach(context, settings) {
      if ($(context)[0].nodeName === '#document') {
        $(
          once(
            'subfolders-folder-icon',
            '#subfolders .folder a.folder-icon',
            context,
          ),
        ).click(function (event) {
          event.preventDefault();
          event.stopPropagation();

          const folder = $(this).parent('.folder');

          $('#subfolders .folder a.selected')
            .not($(this))
            .removeClass('selected');
          $('#subfolders .folder.selected').not(folder).removeClass('selected');

          $('#folders-details span').html('');
          $(this).toggleClass('selected');
          folder.toggleClass('selected');
        });

        $(once('subfolders-navbar', '#navbar a', context)).click(
          function (event) {
            event.preventDefault();
            event.stopPropagation();

            const url = $(this).attr('href');
            $(this).ajaxWidgetLoad(
              url,
              settings.media_folders.state_parameters,
            );
          },
        );

        $(
          once('navbar-expand', '#explorer #navbar > ul li > span', context),
        ).click(function (event) {
          $(this).parent().toggleClass('active-trail');
        });

        $(once('subfolders-folder', '#subfolders .folder', context)).dblclick(
          function (event) {
            event.preventDefault();
            event.stopPropagation();

            if ($(this).find('a').hasClass('folder-icon-file')) {
              const fid = $('#subfolders').attr('data-id');
              $(this).ajaxWidgetUpdate(
                fid,
                $(this).attr('data-id'),
                settings.media_folders,
              );
            } else {
              event.preventDefault();
              event.stopPropagation();
              const url = $(this).find('a').attr('href');
              $(this).ajaxWidgetLoad(
                url,
                settings.media_folders.state_parameters,
              );
            }
          },
        );

        $(window).on('dialogopen', (e, ui) => {
          Drupal.attachBehaviors();
        });
      }
    },
  };

  Drupal.behaviors.MediaFoldersWidgetSortable = {
    attach(context) {
      const selection = context.querySelectorAll('.js-media-folders-selection');
      selection.forEach((widget) => {
        Sortable.create(widget, {
          draggable: '.js-media-folders-item',
          onEnd: () => {
            $(widget)
              .children()
              .each((index, child) => {
                $(child).find('.js-media-folders-item-weight')[0].value = index;
              });
          },
        });
      });
    },
  };

  Drupal.behaviors.MediaFoldersWidgetToggleWeight = {
    attach(context) {
      const strings = {
        show: Drupal.t('Show media item weights'),
        hide: Drupal.t('Hide media item weights'),
      };
      const mediaFoldersToggle = once(
        'media-folders-toggle',
        '.js-media-folders-widget-toggle-weight',
        context,
      );
      $(mediaFoldersToggle).on('click', (e) => {
        e.preventDefault();
        const $target = $(e.currentTarget);
        e.currentTarget.textContent = $target.hasClass('active')
          ? strings.show
          : strings.hide;
        $target
          .toggleClass('active')
          .closest('.js-media-folders-widget')
          .find('.js-media-folders-item-weight')
          .parent()
          .toggle();
      });
      mediaFoldersToggle.forEach((item) => {
        item.textContent = strings.show;
      });

      $(once('media-folders-toggle', '.js-media-folders-item-weight', context))
        .parent()
        .hide();
    },
  };

  Drupal.behaviors.MediaFoldersWidgetDisableButton = {
    attach(context) {
      once(
        'media-folders-disable',
        '.js-media-folders-open-button[data-disabled-focus="true"]',
        context,
      ).forEach((button) => {
        $(button).focus();
        setTimeout(() => {
          $(button).attr('disabled', 'disabled');
        }, 50);
      });
    },
  };
})(jQuery, Drupal, Sortable);
